#!/bin/bash
# Copyright 1999-2005 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
# $Header: /var/cvsroot/gentoo/src/catalyst/livecd/runscript/Attic/default-runscript.sh,v 1.25.2.11 2005/07/28 18:02:52 wolf31o2 Exp $

#return codes to be used by archscript
die() {
	echo "$1"
	exit 1
}

case ${clst_livecd_cdfstype} in
	zisofs)
		cmdline_opts="looptype=zisofs loop=/zisofs"
		;;
	normal)
		cmdline_opts="looptype=normal loop=/livecd.loop"
		;;
	noloop)
		cmdline_opts="looptype=noloop"
		;;
	squashfs)
		cmdline_opts="looptype=squashfs loop=/livecd.squashfs"
		;;
esac

source ${clst_livecd_archscript}
# Here is where we poke in our identifier
touch ${clst_cdroot_path}/livecd
# Here we copy our nifty README.txt
#cp ${clst_sharedir}/livecd/files/README.txt ${clst_cdroot_path}

create_normal_loop() {
	# We get genkernel-built kernels and initrds in place, create the loopback
	# filesystem on $clst_cdroot_path, mount it, copy our bootable filesystem
	# over, umount it, and we then have a ready-to-burn ISO tree at
	# $clst_cdroot_path.

	echo "Calculating size of loopback filesystem..."
	loopsize=`du -ks ${clst_chroot_path} | cut -f1`
	[ "${loopsize}" = "0" ] && loopsize=1
	# Add 4MB for filesystem slop
	loopsize=`expr ${loopsize} + 4096`
	echo "Creating loopback file..."
	dd if=/dev/zero of=${clst_cdroot_path}/livecd.loop bs=1k count=${loopsize} \
		|| die "livecd.loop creation failure"
	mke2fs -m 0 -F -q ${clst_cdroot_path}/livecd.loop \
		|| die "Couldn't create ext2 filesystem"
	install -d ${clst_cdroot_path}/loopmount
	sync; sync; sleep 3 #try to work around 2.6.0+ loopback bug
	mount -t ext2 -o loop ${clst_cdroot_path}/livecd.loop \
		${clst_cdroot_path}/loopmount \
		|| die "Couldn't mount loopback ext2 filesystem"
	sync; sync; sleep 3 #try to work around 2.6.0+ loopback bug
	echo "cp -a ${clst_chroot_path}/* ${clst_cdroot_path}/loopmount"
	cp -a ${clst_chroot_path}/* ${clst_cdroot_path}/loopmount 

	[ $? -ne 0 ] && { umount ${clst_cdroot_path}/loopmount; die "Couldn't copy files to loopback ext2 filesystem"; }
	umount ${clst_cdroot_path}/loopmount || die "Couldn't unmount loopback ext2 filesystem"
	rm -rf ${clst_cdroot_path}/loopmount
}

create_zisofs() {

	rm -rf "${clst_cdroot_path}/zisofs" > /dev/null 2>&1
	echo "Creating zisofs..."
	mkzftree -z 9 -p2 "${clst_chroot_path}" "${clst_cdroot_path}/zisofs" || die "Could not run mkzftree, did you emerge zisofs"

}

create_noloop() {

	echo "Copying files for image (no loop)..."
	cp -a "${clst_chroot_path}"/* "${clst_cdroot_path}" || die "Could not copy files to image (no loop)"
	
}

create_squashfs() {
	echo "Creating squashfs..."
	mksquashfs "${clst_chroot_path}" "${clst_cdroot_path}/livecd.squashfs" -noappend || die "mksquashfs failed, did you emerge squashfs-tools?"
	
}

## START RUNSCRIPT

case $1 in
	kernel)
		shift
		numkernels="$1"
		shift

		# if we have our own linuxrc, copy it in
		if [ -n "${clst_livecd_linuxrc}" ]
		then
			cp -a ${clst_livecd_linuxrc} ${clst_chroot_path}/tmp/linuxrc
		fi

		# setup genkernel and do any pre-kernel merge opts
		cp -a ${clst_sharedir}/livecd/runscript-support/pre-kmerge.sh ${clst_chroot_path}/tmp
		${clst_CHROOT} ${clst_chroot_path} /tmp/pre-kmerge.sh || exit 1
		rm -f ${clst_chroot_path}/tmp/pre-kmerge.sh

		# the kernel merge process is done in a chroot
		cp -a ${clst_sharedir}/livecd/runscript-support/kmerge.sh ${clst_chroot_path}/tmp
		count=0
		while [ ${count} -lt ${numkernels} ]
		do
			export clst_kname="$1"
			shift
			export clst_ksource="$1"
			shift
			export clst_kextversion="$1"
			shift
		${clst_CHROOT} ${clst_chroot_path} /tmp/kmerge.sh || exit 1
		count=$(( ${count} + 1 ))
		done
		rm -f ${clst_chroot_path}/tmp/kmerge.sh
		rm -f ${clst_chroot_path}/tmp/linuxrc
	
		# clean up genkernel and do any post-kernel merge opts
		cp -a ${clst_sharedir}/livecd/runscript-support/post-kmerge.sh ${clst_chroot_path}/tmp
		${clst_CHROOT} ${clst_chroot_path} /tmp/post-kmerge.sh || exit 1
		rm -f ${clst_chroot_path}/tmp/post-kmerge.sh
		;;

	preclean)
		# move over the motd (if applicable)
		if [ -n "${clst_livecd_motd}" ]
		then
			cp -a ${clst_livecd_motd} ${clst_chroot_path}/etc/motd
		else
			cp -a ${clst_sharedir}/livecd/files/generic.motd.txt \
				${clst_sharedir}/livecd/files/universal.motd.txt \
				${clst_sharedir}/livecd/files/minimal.motd.txt \
				${clst_sharedir}/livecd/files/livecd.motd.txt \
				${clst_sharedir}/livecd/files/gamecd.motd.txt \
				${clst_chroot_path}/etc
		fi
	
		# move over the xinitrc (if applicable)
		if [ -n "${clst_livecd_xinitrc}" ]
		then
			cp -a ${clst_livecd_xinitrc} ${clst_chroot_path}/etc/X11/xinit/xinitrc
		fi

		# move over the readme (if applicable)
		if [ -n "${clst_livecd_readme}" ]
		then
			cp -a ${clst_livecd_readme} ${clst_cdroot_path}/README.txt
		else
			cp ${clst_sharedir}/livecd/files/README.txt ${clst_cdroot_path}
		fi

		# move over Getting_Online.txt for minimal/GameCD
		if [ "${clst_livecd_type}" = "gentoo-gamecd" ] \
		|| [ "${clst_livecd_type}" = "gentoo-release-minimal" ] \
		|| [ "${clst_livecd_type}" = "gentoo-release-livecd" ]
		then
			cp ${clst_sharedir}/livecd/files/Getting_Online.txt ${clst_cdroot_path}
		fi

		# move over the environment
		cp ${clst_sharedir}/livecd/files/livecd-bashrc \
			${clst_chroot_path}/root/.bashrc
		cp ${clst_sharedir}/livecd/files/livecd-bash_profile \
			${clst_chroot_path}/root/.bash_profile
		cp ${clst_sharedir}/livecd/files/livecd-local.start \
			${clst_chroot_path}/etc/conf.d/local.start

		# touch /etc/startx if our livecd/type requires it
		if [ "${clst_livecd_type}" = "gentoo-gamecd" ] \
		|| [ "${clst_livecd_type}" = "generic-livecd" ] #\
		#|| [ "${clst_livecd_type}" = "gentoo-release-livecd" ]
		then
			touch ${clst_chroot_path}/etc/startx
		fi
		
		# now, finalize and tweak the livecd fs (inside of the chroot)
		cp ${clst_sharedir}/livecd/runscript-support/livecdfs-update.sh ${clst_chroot_path}/tmp
		${clst_CHROOT} ${clst_chroot_path} /tmp/livecdfs-update.sh || exit 1
		rm -f ${clst_chroot_path}/tmp/livecdfs-update.sh

		# execute gamecdfs-update.sh if we're a gamecd
		if [ "${clst_livecd_type}" = "gentoo-gamecd" ]
		then
			cp ${clst_sharedir}/livecd/runscript-support/gamecdfs-update.sh ${clst_chroot_path}/tmp
			cp ${clst_gamecd_conf} ${clst_chroot_path}/tmp/gamecd.conf
			${clst_CHROOT} ${clst_chroot_path} /tmp/gamecdfs-update.sh || exit 1
			rm -f ${clst_chroot_path}/tmp/gamecdfs-update.sh \
				${clst_chroot_path}/tmp/gamecd.conf
		fi
		
		# if the user has their own fs update script, execute it
		if [ -n "${clst_livecd_fsscript}" ]
		then
			cp ${clst_livecd_fsscript} ${clst_chroot_path}/tmp/fsscript.sh
			chmod 755 ${clst_chroot_path}/tmp/fsscript.sh
			${clst_CHROOT} ${clst_chroot_path} /tmp/fsscript.sh || exit 1
			rm -f ${clst_chroot_path}/tmp/fsscript.sh
		fi
		;;

	clean)
		find ${clst_chroot_path}/usr/lib -iname "*.pyc" -exec rm -f {} \;
		;;

	bootloader)
		;;

	cdfs)
		loopret=1
		if [ "${clst_livecd_cdfstype}" = "normal" ]
		then
			create_normal_loop
			loopret=$?
		elif [ "${clst_livecd_cdfstype}" = "zisofs" ]
		then
			create_zisofs
			loopret=$?
		elif [ "${clst_livecd_cdfstype}" = "noloop" ]
		then
			create_noloop
			loopret=$?
		elif [ "${clst_livecd_cdfstype}" = "squashfs" ]
		then
			create_squashfs
			loopret=$?
		fi		
		exit $loopret
		;;

	iso)
		;;
esac
exit 0 
